@@ -15,6 +15,8 @@ import numpy
 import uncertainties
 from uncertainties import ufloat
 from uncertainties import unumpy
+from . import core
+
 from .. import test_uncertainties
 from uncertainties.test_uncertainties import _numbers_close, matrices_close
 
@@ -78,21 +80,8 @@ def test_matrix():
     # Test of the nominal_value attribute:
     assert numpy.all(m_nominal_values == m.nominal_values)
 
-    # "Regular" inverse matrix, when uncertainties are not taken
-    # into account:
-    m_no_uncert_inv = m_nominal_values.I
     assert type(m[0, 0]) == uncertainties.Variable
 
-    # The matrix inversion should not yield numbers with uncertainties:
-    assert m_no_uncert_inv.dtype == numpy.dtype(float)
-
-    # Inverse with uncertainties:
-    m_inv_uncert = m.I  # AffineScalarFunc elements
-    # The inverse contains uncertainties: it must support custom
-    # operations on matrices with uncertainties:
-    assert isinstance(m_inv_uncert, unumpy.matrix)
-    assert type(m_inv_uncert[0, 0]) == uncertainties.AffineScalarFunc
-
     # Checks of the numerical values: the diagonal elements of the
     # inverse should be the inverses of the diagonal elements of
     # m (because we started with a triangular matrix):
@@ -103,9 +92,45 @@ def test_matrix():
     assert _numbers_close(1/m_nominal_values[1, 1],
                           m_inv_uncert[1, 1].nominal_value), "Wrong value"
 
+def _derivatives_close(x, y):
+    """
+    Returns True iff the AffineScalarFunc objects x and y have
+    derivatives that are close to each other (they must depend
+    on the same variables).
+    """
+
+    # x and y must depend on the same variables:
+    if len(set(x.derivatives)\
+           .symmetric_difference(y.derivatives)):
+        return False  # Not the same variables
+
+    return all( _numbers_close(x.derivatives[var],
+                               y.derivatives[var])
+                for var in x.derivatives)
+
 def test_inverse():
     "Tests of the matrix inverse"
 
+    m = unumpy.matrix([[ufloat((10, 1)), -3.1],
+                       [0, ufloat((3, 0))]])
+    m_nominal_values = unumpy.nominal_values(m)
+
+    # "Regular" inverse matrix, when uncertainties are not taken
+    # into account:
+    m_no_uncert_inv = m_nominal_values.I
+
+    # The matrix inversion should not yield numbers with uncertainties:
+    assert m_no_uncert_inv.dtype == numpy.dtype(float)
+
+    # Inverse with uncertainties:
+    m_inv_uncert = m.I  # AffineScalarFunc elements
+    # The inverse contains uncertainties: it must support custom
+    # operations on matrices with uncertainties:
+    assert isinstance(m_inv_uncert, unumpy.matrix)
+    assert type(m_inv_uncert[0, 0]) == uncertainties.AffineScalarFunc
+
+    ####################
+
     # Checks of the covariances between elements:
     x = ufloat((10, 1))
     m = unumpy.matrix([[x, x],
@@ -122,23 +147,6 @@ def test_inverse():
     assert _numbers_close(m_double_inverse[0, 0].std_dev(),
                           m[0, 0].std_dev())
 
-
-    def _derivatives_close(x, y):
-        """
-        Returns True iff the AffineScalarFunc objects x and y have
-        derivatives that are close to each other (they must depend
-        on the same variables).
-        """
-
-        # x and y must depend on the same variables:
-        if len(set(x.derivatives)\
-               .symmetric_difference(y.derivatives)):
-            return False  # Not the same variables
-
-        return all( _numbers_close(x.derivatives[var],
-                                   y.derivatives[var])
-                    for var in x.derivatives)
-
     assert matrices_close(m_double_inverse, m)
 
     # Partial test:
@@ -158,6 +166,40 @@ def test_inverse():
     # inversion:
     assert matrices_close(m * m_inverse,  numpy.eye(m.shape[0]))
 
+def test_pseudo_inverse():
+    "Tests of the pseudo-inverse"
+
+    # Numerical version of the pseudo-inverse:
+    pinv_num = core.wrap_array_func(numpy.linalg.pinv)
+
+    ##########
+    # Full rank rectangular matrix:
+    m = unumpy.matrix([[ufloat((10, 1)), -3.1],
+                       [0, ufloat((3, 0))],
+                       [1, -3.1]])
+
+    # Numerical and package (analytical) pseudo-inverses: they must be
+    # the same:
+    rcond = 1e-8  # Test of the second argument to pinv()
+    m_pinv_num = pinv_num(m, rcond)
+    m_pinv_package = core._pinv(m, rcond)
+    assert matrices_close(m_pinv_num, m_pinv_package)
+
+    ##########
+    # Example with a non-full rank rectangular matrix:
+    vector = [ufloat((10, 1)), -3.1, 11]
+    m = unumpy.matrix([vector, vector])
+    m_pinv_num = pinv_num(m, rcond)
+    m_pinv_package = core._pinv(m, rcond)
+    assert matrices_close(m_pinv_num, m_pinv_package)
+    
+    ##########
+    # Example with a non-full-rank square matrix:
+    m = unumpy.matrix([[ufloat((10, 1)), 0], [3, 0]])
+    m_pinv_num = pinv_num(m, rcond)
+    m_pinv_package = core._pinv(m, rcond)
+    assert matrices_close(m_pinv_num, m_pinv_package)
+    
 def test_broadcast_funcs():
     """
     Test of mathematical functions that work with NumPy arrays of
